// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/driver/device.c $
// $Rev: 53633 $
// $Date: 2023-08-29 09:41:27 -0500 (Tue, 29 Aug 2023) $

// 14HSAI4: Device Driver: source file

#include "main.h"



// variables ******************************************************************

const gsc_dev_id_t	dev_id_list[]	=
{
	// model		Vendor	Device	SubVen	SubDev	type
	{ "14HSAI4",	0x10B5, 0x9056, 0x10B5, 0x3300,	GSC_DEV_TYPE_14HSAI4	},
	{ NULL }
};



//*****************************************************************************
static void _range_compute(dev_data_t* dev)
{
	switch (dev->cache.gsc_bcfgr_32 & 0x60000)
	{
		default:
		case 0x60000:	// reserved
		case 0x00000:	// High-Level inputs

			dev->cache.range_max	= HSAI4_AI_RANGE_10V;
			dev->cache.range_min	= HSAI4_AI_RANGE_2_5V;
			break;

		case 0x20000:	// Wideband inputs

			dev->cache.range_max	= HSAI4_AI_RANGE_2_5V;
			dev->cache.range_min	= HSAI4_AI_RANGE_0_625V;
			break;

		case 0x40000:	// Transformer-Coupled inputs

			dev->cache.range_max	= HSAI4_AI_RANGE_2_0VPP;
			dev->cache.range_min	= HSAI4_AI_RANGE_1_0VPP;
			break;
	}
}



/******************************************************************************
*
*	Function:	dev_device_create
*
*	Purpose:
*
*		Do everything needed to setup and use the given device.
*
*	Arguments:
*
*		dev		The structure to initialize.
*
*	Returned:
*
*		0		All is well.
*		< 0		An appropriate error status.
*
******************************************************************************/

int dev_device_create(dev_data_t* dev)
{
	static const gsc_bar_maps_t	bar_map	=
	{
		{
			// mem	io	rw
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 0: PLX registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 1: PLX registers, I/O mapped
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RW },	// BAR 2: GSC registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 3: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 4: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO }	// BAR 5: unused
		}
	};

	u32	dma;
	int	ret;

	for (;;)	// A convenience loop.
	{
		// Verify some macro contents.
		ret	= gsc_macro_test_base_name(HSAI4_BASE_NAME);
		if (ret)	break;

		ret	= gsc_macro_test_model();
		if (ret)	break;

		// PCI setup.
		ret	= os_pci_dev_enable(&dev->pci);
		if (ret)	break;

		ret	= os_pci_master_set(&dev->pci);
		if (ret)	break;

		// Control ISR access to the device and data structure.
		ret	= os_spinlock_create(&dev->spinlock);
		if (ret)	break;

		// Control access to the device and data structure.
		ret	= os_sem_create(&dev->sem);
		if (ret)	break;

		// Access the BAR regions.
		ret	= gsc_bar_create(dev, &dev->bar, &bar_map);
		if (ret)	break;

		// Firmware access.
		dev->vaddr.gsc_bcfgr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HSAI4_GSC_BCFGR));
		dev->vaddr.gsc_bctlr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HSAI4_GSC_BCTLR));
		dev->vaddr.gsc_bsizr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HSAI4_GSC_BSIZR));
		dev->vaddr.gsc_bufsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HSAI4_GSC_BUFSR));
		dev->vaddr.gsc_cscr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HSAI4_GSC_CSCR));
		dev->vaddr.gsc_dfr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HSAI4_GSC_DFR));
		dev->vaddr.gsc_ibcr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HSAI4_GSC_IBCR));
		dev->vaddr.gsc_idbr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HSAI4_GSC_IDBR));
		dev->vaddr.gsc_psizr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HSAI4_GSC_PSIZR));
		dev->vaddr.gsc_pstsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HSAI4_GSC_PSTSR));
		dev->vaddr.gsc_rdr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HSAI4_GSC_RDR));
		dev->vaddr.gsc_rgar_32		= GSC_VADDR(dev, GSC_REG_OFFSET(HSAI4_GSC_RGAR));

		// Data cache initialization.
		dev->cache.gsc_bcfgr_32		= os_reg_mem_rx_u32(NULL, dev->vaddr.gsc_bcfgr_32);

		dev->cache.ai_chan_qty		= (dev->cache.gsc_bcfgr_32 & 0x10000) ? 2 : 4;
		dev->cache.ai_chans_max		= 4;
		dev->cache.autocal			= (dev->cache.gsc_bcfgr_32 & 0x40000) ? 0 : 1;
		dev->cache.autocal_ms		= 6000;			// device manual section 3.6

		dev->cache.fgen_max			= 50000000L;	// device manual section 3.4.3.2
		dev->cache.fgen_min			= 25000000L;	// device manual section 3.4.3.2
		dev->cache.fifo_size_rx		= _256K;		// 32-bits wide
		dev->cache.fref_default		= 30000000L;	// device manual section 3.4.3.2
		dev->cache.fsamp_agg		= 100000000L;	// device manual section 1.1
		dev->cache.fsamp_max		= 50000000L;	// device manual section 3.4.3.2
		dev->cache.fsamp_min		= 1000;			// device manual section 3.4.3.2
		dev->cache.fsync_max		= 10000000;		// device manual section 3.4.4.1
		dev->cache.fsync_min		= 24;			// device manual section 3.4.4.1

		dev->cache.initialize_ms	= 20;			// device manual section 3.3.2
		dev->cache.input_config		= (dev->cache.gsc_bcfgr_32 & 0x60000) >> 17;
		dev->cache.input_impedence	= (dev->cache.gsc_bcfgr_32 & 0x80000) ? 1 : 0;

		dev->cache.ndec_max			= 1000;			// device manual section 3.4.3.2
		dev->cache.ndec_min			= 1;			// device manual section 3.4.3.2
		dev->cache.ndiv_clk_max		= 25;			// device manual section 3.4.3.2
		dev->cache.ndiv_clk_min		= 1;			// device manual section 3.4.3.2
		dev->cache.ndiv_sync_max	= 0xFFFFF;		// device manual section 3.4.4.1
		dev->cache.ndiv_sync_min	= 10;			// device manual section 3.4.4.1
		dev->cache.nref_max			= 1000;			// device manual section 3.4.3.2
		dev->cache.nref_min			= 20;			// device manual section 3.4.3.2
		dev->cache.nvco_max			= 1000;			// device manual section 3.4.3.2
		dev->cache.nvco_min			= 20;			// device manual section 3.4.3.2

		dev->cache.rate_gen_ms		= 20;			// device manual section 3.4.3.2

		_range_compute(dev);

		// Initialize additional resources.
		ret	= dev_irq_create(dev);
		if (ret)	break;

		ret	= dev_io_create(dev);
		if (ret)	break;

		dma	= GSC_DMA_SEL_STATIC
			| GSC_DMA_CAP_BMDMA_READ
			| GSC_DMA_CAP_DMDMA_READ;
		ret	= gsc_dma_create(dev, dma, dma);
		break;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	dev_device_destroy
*
*	Purpose:
*
*		Do everything needed to release the referenced device.
*
*	Arguments:
*
*		dev		The partial data for the device of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_device_destroy(dev_data_t* dev)
{
	if (dev)
	{
		gsc_dma_destroy(dev);
		dev_io_destroy(dev);
		dev_irq_destroy(dev);
		gsc_bar_destroy(&dev->bar);
		os_sem_destroy(&dev->sem);
		os_spinlock_destroy(&dev->spinlock);
		os_pci_master_clear(&dev->pci);
		os_pci_dev_disable(&dev->pci);
	}
}


