// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/driver/ioctl.c $
// $Rev: 53633 $
// $Date: 2023-08-29 09:41:27 -0500 (Tue, 29 Aug 2023) $

// 14HSAI4: Device Driver: source file

#include "main.h"



// macros *********************************************************************

#define	AUTOCAL_PASS				0x4000
#define	AUTOCAL_START				0x2000
#define	INIT_START					0x8000

#define	PSTSR_IRQ_AUTOCAL_DONE		HSAI4_IRQ_AUTOCAL_DONE



//*****************************************************************************
static void _wait_for_ready(dev_data_t* dev)
{
	// Wait for the ready bit to become not ready.
	os_time_sleep_ms(2);

	// Now wait for the bit to become ready.
	gsc_poll_u32(dev, dev->cache.rate_gen_ms + 50, dev->vaddr.gsc_cscr_32, 0x20, 0);
}



//*****************************************************************************
static int _query(dev_data_t* dev, s32* arg)
{
	switch (arg[0])
	{
		default:							arg[0]	= HSAI4_IOCTL_QUERY_ERROR;		break;
		case HSAI4_QUERY_AI_RANGE_MAX:		arg[0]	= dev->cache.range_max;			break;
		case HSAI4_QUERY_AI_RANGE_MIN:		arg[0]	= dev->cache.range_min;			break;
		case HSAI4_QUERY_AUTOCAL:			arg[0]	= dev->cache.autocal;			break;
		case HSAI4_QUERY_AUTOCAL_MS:		arg[0]	= dev->cache.autocal_ms;		break;
		case HSAI4_QUERY_CHANNEL_AI_MAX:	arg[0]	= dev->cache.ai_chans_max;		break;
		case HSAI4_QUERY_CHANNEL_AI_QTY:	arg[0]	= dev->cache.ai_chan_qty;		break;
		case HSAI4_QUERY_COUNT:				arg[0]	= HSAI4_IOCTL_QUERY_LAST;		break;
		case HSAI4_QUERY_DEVICE_TYPE:		arg[0]	= dev->board_type;				break;
		case HSAI4_QUERY_FGEN_MAX:			arg[0]	= dev->cache.fgen_max;			break;
		case HSAI4_QUERY_FGEN_MIN:			arg[0]	= dev->cache.fgen_min;			break;
		case HSAI4_QUERY_FIFO_SIZE_RX:		arg[0]	= dev->cache.fifo_size_rx;		break;
		case HSAI4_QUERY_FREF_DEFAULT:		arg[0]	= dev->cache.fref_default;		break;
		case HSAI4_QUERY_FSAMP_AGG:			arg[0]	= dev->cache.fsamp_agg;			break;
		case HSAI4_QUERY_FSAMP_MAX:			arg[0]	= dev->cache.fsamp_max;			break;
		case HSAI4_QUERY_FSAMP_MIN:			arg[0]	= dev->cache.fsamp_min;			break;
		case HSAI4_QUERY_FSYNC_MAX:			arg[0]	= dev->cache.fsync_max;			break;
		case HSAI4_QUERY_FSYNC_MIN:			arg[0]	= dev->cache.fsync_min;			break;
		case HSAI4_QUERY_INIT_MS:			arg[0]	= dev->cache.initialize_ms;		break;
		case HSAI4_QUERY_INPUT_CONFIG:		arg[0]	= dev->cache.input_config;		break;
		case HSAI4_QUERY_INPUT_IMPED:		arg[0]	= dev->cache.input_impedence;	break;
		case HSAI4_QUERY_NDEC_MAX:			arg[0]	= dev->cache.ndec_max;			break;
		case HSAI4_QUERY_NDEC_MIN:			arg[0]	= dev->cache.ndec_min;			break;
		case HSAI4_QUERY_NDIV_CLK_MAX:		arg[0]	= dev->cache.ndiv_clk_max;		break;
		case HSAI4_QUERY_NDIV_CLK_MIN:		arg[0]	= dev->cache.ndiv_clk_min;		break;
		case HSAI4_QUERY_NDIV_SYNC_MAX:		arg[0]	= dev->cache.ndiv_sync_max;		break;
		case HSAI4_QUERY_NDIV_SYNC_MIN:		arg[0]	= dev->cache.ndiv_sync_min;		break;
		case HSAI4_QUERY_NREF_MAX:			arg[0]	= dev->cache.nref_max;			break;
		case HSAI4_QUERY_NREF_MIN:			arg[0]	= dev->cache.nref_min;			break;
		case HSAI4_QUERY_NVCO_MAX:			arg[0]	= dev->cache.nvco_max;			break;
		case HSAI4_QUERY_NVCO_MIN:			arg[0]	= dev->cache.nvco_min;			break;
		case HSAI4_QUERY_RATE_GEN_MS:		arg[0]	= dev->cache.rate_gen_ms;		break;
	}

	return(0);
}



//*****************************************************************************
int initialize_ioctl(dev_data_t* dev, void* arg)
{
	int			i;
	long		ms		= dev->cache.initialize_ms + 5000;
	u32			reg;
	int			ret;
	VADDR_T		va		= dev->vaddr.gsc_bctlr_32;

	// This device doesn't have an Initialization Done interrupt so we simply
	// initiate initialization and wait for the Init Start bit to clear.

	// Initiate initialization.
	os_reg_mem_tx_u32(dev, va, INIT_START);

	// Manually wait for completion.
	ret	= gsc_poll_u32(dev, ms, va, INIT_START, 0);

	if (ret)
	{
		printf(	"%s: INITIALIZATION DID NOT COMPLETE WITHIN %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually check for completion as the IRQ wait may have ended early.
	reg	= os_reg_mem_rx_u32(dev, va);

	if (reg & INIT_START)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: INITILIZE STILL ACTIVE AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Initialize the software settings.

	for (i = 0; i < DEV_IO_STREAM_QTY; i++)
	{
		if (dev->io.io_streams[i])
		{
			if (dev->io.io_streams[i]->dev_io_sw_init)
			{
				(dev->io.io_streams[i]->dev_io_sw_init)(dev, dev->io.io_streams[i]);
			}
		}
	}

	return(ret);
}



//*****************************************************************************
static int _autocal_start(dev_data_t* dev, void* arg)
{
	// Initiate autocalibration.
	os_reg_mem_mx_u32(NULL, dev->vaddr.gsc_bctlr_32, AUTOCAL_START, AUTOCAL_START);
	return(0);
}



//*****************************************************************************
static int _autocal(dev_data_t* dev, void* arg)
{
	long		ms		= dev->cache.autocal_ms + 5000;
	u32			reg;
	int			ret		= 0;
	os_sem_t	sem;
	int			tmp;
	VADDR_T		va		= dev->vaddr.gsc_pstsr_32;
	gsc_wait_t	wt;

	if (dev->cache.autocal)
	{
		// Safely select the Autocal Done interrupt.
		os_reg_mem_tx_u32(dev, va, PSTSR_IRQ_AUTOCAL_DONE);

		// Wait for the local interrupt.
		os_sem_create(&sem);	// dummy, required for wait operations.
		memset(&wt, 0, sizeof(wt));
		wt.flags		= GSC_WAIT_FLAG_INTERNAL;
		wt.gsc			= HSAI4_WAIT_GSC_AUTOCAL_DONE;
		wt.timeout_ms	= ms;
		ret				= gsc_wait_event(dev, &wt, _autocal_start, NULL, &sem);
		os_sem_destroy(&sem);

		if (wt.flags & GSC_WAIT_FLAG_TIMEOUT)
		{
			ret	= ret ? ret : -ETIMEDOUT;
			printf(	"%s: AUTOCALIBRATION TIMED OUT AFTER %ld ms.\n",
					dev->model,
					ms);
		}

		va		= dev->vaddr.gsc_bctlr_32;

		// Manually wait for completion in case something terminates our wait early.
		tmp	= gsc_poll_u32(dev, ms, va, AUTOCAL_START, 0);

		if (tmp)
		{
			ms	*= 2;
			ret	= ret ? ret : tmp;
			printf(	"%s: AUTOCALIBRATION DID NOT COMPLETE WITHIN %ld ms.\n",
					dev->model,
					ms);
		}

		// Manually check for completion as the IRQ wait may have ended early.
		reg	= os_reg_mem_rx_u32(dev, va);

		if (reg & AUTOCAL_START)
		{
			ret	= ret ? ret : -ETIMEDOUT;
			printf(	"%s: AUTOCALIBRATION STILL ACTIVE AFTER %ld ms.\n",
					dev->model,
					ms);
		}

		// Final results.
		reg	= os_reg_mem_rx_u32(dev, va);

		if ((reg & AUTOCAL_PASS) == 0)
		{
			ret	= ret ? ret : -EIO;
			printf(	"%s: AUTOCALIBRATION FAILED (%ld ms).\n",
					dev->model,
					ms);
		}

		if (ret == 0)
		{
			// Wait for settling.
			os_time_sleep_ms(100);
		}
	}

	return(ret);
}



//*****************************************************************************
static int _autocal_status(dev_data_t* dev, s32* arg)
{
	u32	reg;
	int	ret;
	s32	value;

	if (dev->cache.autocal)
	{
		ret	= 0;
		reg	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bctlr_32);

		if (reg & AUTOCAL_START)
			value	= HSAI4_AUTOCAL_STATUS_ACTIVE;
		else if (reg & AUTOCAL_PASS)
			value	= HSAI4_AUTOCAL_STATUS_PASS;
		else
			value	= HSAI4_AUTOCAL_STATUS_FAIL;

		arg[0]	= value;
	}
	else
	{
		// Autocalibration is not supported on this device.
		ret	= -EIO;
	}

	return(ret);
}



//*****************************************************************************
static int _rx_io_mode(dev_data_t* dev, s32* arg)
{
	static const s32	list[]	=
	{
		GSC_IO_MODE_PIO,
		GSC_IO_MODE_BMDMA,
		GSC_IO_MODE_DMDMA,
		-1
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, list, &dev->io.rx.io_mode);
	return(ret);
}



//*****************************************************************************
static int _rx_io_overflow(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSAI4_IO_OVERFLOW_IGNORE,
		HSAI4_IO_OVERFLOW_CHECK,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.rx.overflow_check);
	return(ret);
}



//*****************************************************************************
static int _rx_io_timeout(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_var(
			arg,
			HSAI4_IO_TIMEOUT_MIN,
			HSAI4_IO_TIMEOUT_INFINITE,
			&dev->io.rx.timeout_s);
	return(ret);
}



//*****************************************************************************
static int _rx_io_underflow(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSAI4_IO_UNDERFLOW_IGNORE,
		HSAI4_IO_UNDERFLOW_CHECK,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.rx.underflow_check);
	return(ret);
}



//*****************************************************************************
static int _ai_buf_clear(dev_data_t* dev, void* arg)
{
	#define	CLEAR	D18		// IBCR
	#define	OVER	D17		// BCTLR
	#define	UNDER	D16		// BCTLR

	int		i;
	u32		reg;
	int		ret	= 0;
	VADDR_T	va	= dev->vaddr.gsc_ibcr_32;

	// Clear the buffer.
	os_reg_mem_mx_u32(dev, va, CLEAR, CLEAR);

	// Wait for the bit to clear.

	for (i = 0;; i++)
	{
		reg	= os_reg_mem_rx_u32(dev, va);

		if ((reg & CLEAR) == 0)
			break;

		if (i >= 250)
		{
			ret	= -EINVAL;
			printf(	"%s: The analog input buffer took too long to clear.\n",
					DEV_NAME);
			break;
		}
	}

	va	= dev->vaddr.gsc_bcfgr_32;

	// Clear the overflow flag.
	os_reg_mem_mx_u32(dev, va, 0, OVER);

	// Clear the underflow flag.
	os_reg_mem_mx_u32(dev, va, 0, UNDER);

	return(ret);
}



//*****************************************************************************
static int _ai_buf_enable(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSAI4_AI_BUF_ENABLE_NO,
		HSAI4_AI_BUF_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_ibcr_32, 20, 20);
	return(ret);
}



//*****************************************************************************
static int _ai_buf_level(dev_data_t* dev, s32* arg)
{
	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_range_reg(dev, arg, 0, 0x7FFFF, dev->vaddr.gsc_bufsr_32, 18, 0);
	return(ret);
}



//*****************************************************************************
static int _ai_buf_overflow(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSAI4_BUF_ERROR_NO,
		HSAI4_BUF_ERROR_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 17, 17);
	return(ret);
}



//*****************************************************************************
static int _ai_buf_thr_lvl(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0x3FFFF, dev->vaddr.gsc_ibcr_32, 17, 0);
	return(ret);
}



//*****************************************************************************
static int _ai_buf_thr_sts(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSAI4_AI_BUF_THR_STS_CLEAR,
		HSAI4_AI_BUF_THR_STS_SET,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_ibcr_32, 19, 19);
	return(ret);
}



//*****************************************************************************
static int _ai_buf_underflow(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSAI4_BUF_ERROR_NO,
		HSAI4_BUF_ERROR_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 16, 16);
	return(ret);
}



//*****************************************************************************
static int _ai_burst_marker(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSAI4_BURST_MARKER_DISABLE,
		HSAI4_BURST_MARKER_ENABLE,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 10, 10);
	return(ret);
}



//*****************************************************************************
static int _ai_burst_mode(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSAI4_BURST_MODE_DISABLE,
		HSAI4_BURST_MODE_ENABLE,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 9, 9);
	return(ret);
}



//*****************************************************************************
static int _ai_burst_single(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSAI4_BURST_SINGLE_DISABLE,
		HSAI4_BURST_SINGLE_ENABLE,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 11, 11);
	return(ret);
}



//*****************************************************************************
static int _ai_burst_size(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFFF, dev->vaddr.gsc_bsizr_32, 19, 0);
	return(ret);
}



//*****************************************************************************
static int _ai_burst_status(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSAI4_BURST_STATUS_IDLE,
		HSAI4_BURST_STATUS_ACTIVE,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 12, 12);
	return(ret);
}



//*****************************************************************************
static int _ai_burst_trigger(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(1L << SHIFT)
	#define	SHIFT	12

	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bctlr_32, MASK, MASK);
	return(0);
}



//*****************************************************************************
static int _ai_chan_sel(dev_data_t* dev, void* arg)
{
	int	ret;

	if (dev->cache.ai_chan_qty == 4)
		ret	= gsc_s32_range_reg(dev, arg, 0, 0xF, dev->vaddr.gsc_cscr_32, 3, 0);
	else
		ret	= gsc_s32_range_reg(dev, arg, 0, 0x3, dev->vaddr.gsc_cscr_32, 1, 0);

	return(ret);
}



//*****************************************************************************
static int _ai_chan_tag(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSAI4_AI_CHAN_TAG_DISABLE,
		HSAI4_AI_CHAN_TAG_ENABLE,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_ibcr_32, 21, 21);
	return(ret);
}



//*****************************************************************************
static int _ai_clock_src(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSAI4_AI_CLOCK_SRC_INTERNAL,
		HSAI4_AI_CLOCK_SRC_EXTERNAL,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_cscr_32, 4, 4);

	// Wait for settling time.
	os_time_sleep_ms(20);
	return(ret);
}



//*****************************************************************************
static int _ai_clocking(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSAI4_AI_CLOCKING_DISABLE,
		HSAI4_AI_CLOCKING_ENABLE,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 8, 8);

	// Wait for settling time.
	os_time_sleep_ms(20);
	return(ret);
}



//*****************************************************************************
static int _ai_data_packing(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSAI4_AI_DATA_PACKING_DISABLE,
		HSAI4_AI_DATA_PACKING_ENABLE,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 18, 18);
	return(ret);
}



//*****************************************************************************
static int _ai_mode(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSAI4_AI_MODE_ZERO,
		HSAI4_AI_MODE_VREF,
		HSAI4_AI_MODE_SYSTEM,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.input_config == HSAI4_INPUT_CONFIG_TRANSFORMER)
		ret	= gsc_s32_list_reg(dev, arg, options + 2, dev->vaddr.gsc_bctlr_32, 2, 0);
	else
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 2, 0);

	return(ret);
}



//*****************************************************************************
static int _ai_pretrig_mode(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSAI4_AI_PRETRIG_MODE_DISABLE,
		HSAI4_AI_PRETRIG_MODE_ENABLE,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_cscr_32, 12, 12);
	return(ret);
}



//*****************************************************************************
static int _ai_pretrig_size(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 2, 0x3FFFF, dev->vaddr.gsc_psizr_32, 17, 0);
	return(ret);
}



//*****************************************************************************
static int _ai_pretrig_sts(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSAI4_AI_PRETRIG_STS_DONE,
		HSAI4_AI_PRETRIG_STS_WAITING,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_cscr_32, 13, 13);
	return(ret);
}



//*****************************************************************************
static int _ai_range(dev_data_t* dev, s32* arg)
{
	typedef struct
	{
		u32		field;
		s32		option;
	} range_t;

	static const range_t	high_level[]	=
	{
		{ 0,	HSAI4_AI_RANGE_2_5V	},
		{ 1,	HSAI4_AI_RANGE_5V	},
		{ 2,	HSAI4_AI_RANGE_10V	},
		{ 3,	HSAI4_AI_RANGE_10V	}
	};

	static const range_t	wideband[]	=
	{
		{ 0,	HSAI4_AI_RANGE_0_625V	},
		{ 1,	HSAI4_AI_RANGE_1_25V	},
		{ 2,	HSAI4_AI_RANGE_2_5V		},
		{ 3,	HSAI4_AI_RANGE_2_5V		}
	};

	static const range_t	transformer[]	=
	{
		{ 0,	HSAI4_AI_RANGE_1_0VPP	},
		{ 1,	HSAI4_AI_RANGE_2_0VPP	},
		{ 2,	HSAI4_AI_RANGE_2_0VPP	},
		{ 3,	HSAI4_AI_RANGE_2_0VPP	}
	};

	static const range_t*	list[4]	= { high_level, wideband, transformer, wideband };

	int				config	= (dev->cache.gsc_bcfgr_32 & 0x60000) >> 17;
	int				i;
	u32				reg;
	int				ret		= 0;
	const range_t*	table	= list[config];
	u32				val;

	if (arg[0] == -1)
	{
		reg		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bctlr_32);
		i		= (reg & 0x30) >> 4;
		arg[0]	= table[i].option;
	}
	else if ((arg[0] < dev->cache.range_min) || (arg[0] > dev->cache.range_max))
	{
		ret	= -EINVAL;
	}
	else
	{
		ret	= -EINVAL;

		for (i = 0; i <= 3; i++)
		{
			if (table[i].option == arg[0])
			{
				ret	= 0;
				val	= table[i].field << 4;
				os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bctlr_32, val, 0x30);
				break;
			}
		}
	}

	return(ret);
}



//*****************************************************************************
static int _ai_sol_chan(dev_data_t* dev, void* arg)
{
	int	ret;

	if (dev->cache.ai_chan_qty == 2)
		ret	= gsc_s32_range_reg(dev, arg, 0, 1, dev->vaddr.gsc_cscr_32, 7, 6);
	else
		ret	= gsc_s32_range_reg(dev, arg, 0, 3, dev->vaddr.gsc_cscr_32, 7, 6);

	return(ret);
}



//*****************************************************************************
static int _ai_sol_holdoff(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSAI4_AI_SOL_HOLDOFF_8,
		HSAI4_AI_SOL_HOLDOFF_64,
		HSAI4_AI_SOL_HOLDOFF_512,
		HSAI4_AI_SOL_HOLDOFF_INF,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_cscr_32, 17, 16);
	return(ret);
}



//*****************************************************************************
static int _ai_sol_int(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSAI4_AI_SOL_INT_1,
		HSAI4_AI_SOL_INT_4,
		HSAI4_AI_SOL_INT_16,
		HSAI4_AI_SOL_INT_64,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_cscr_32, 15, 14);
	return(ret);
}



//*****************************************************************************
static int _ai_sol_level(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0x3FFF, dev->vaddr.gsc_cscr_32, 31, 18);
	return(ret);
}



//*****************************************************************************
static int _ai_sync_src(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSAI4_AI_SYNC_SRC_INTERNAL,
		HSAI4_AI_SYNC_SRC_EXTERNAL,
		HSAI4_AI_SYNC_SRC_SOL_ABOVE,
		HSAI4_AI_SYNC_SRC_SOL_BELOW,
		HSAI4_AI_SYNC_SRC_BCR,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_cscr_32, 10, 8);
	return(ret);
}



//*****************************************************************************
static int _cbl_clk_sync_dir(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSAI4_CBL_CLK_SYNC_DIR_INPUT,
		HSAI4_CBL_CLK_SYNC_DIR_OUTPUT,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_cscr_32, 11, 11);
	return(ret);
}



//*****************************************************************************
static int _data_format(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSAI4_DATA_FORMAT_2S_COMP,
		HSAI4_DATA_FORMAT_OFF_BIN,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 6, 6);
	return(ret);
}



//*****************************************************************************
static int _gen_int_clk_status(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSAI4_GEN_INT_CLK_STATUS_READY,
		HSAI4_GEN_INT_CLK_STATUS_BUSY,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_cscr_32, 5, 5);
	return(ret);
}



//*****************************************************************************
static int _gen_ndec(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 1, 1000, dev->vaddr.gsc_dfr_32, 9, 0);

	// Wait for settling time.
	_wait_for_ready(dev);

	return(ret);
}



//*****************************************************************************
static int _gen_ndiv_clk(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 1, 25, dev->vaddr.gsc_rdr_32, 28, 24);

	// Wait for settling time.
	_wait_for_ready(dev);

	return(ret);
}



//*****************************************************************************
static int _gen_ndiv_sync(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 10, 0xFFFFF, dev->vaddr.gsc_rdr_32, 19, 0);

	// Wait for settling time.
	_wait_for_ready(dev);

	return(ret);
}



//*****************************************************************************
static int _gen_nref(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 20, 1000, dev->vaddr.gsc_rgar_32, 31, 16);

	// Wait for settling time.
	_wait_for_ready(dev);

	return(ret);
}



//*****************************************************************************
static int _gen_nvco(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 20, 1000, dev->vaddr.gsc_rgar_32, 15, 0);

	// Wait for settling time.
	_wait_for_ready(dev);

	return(ret);
}



//*****************************************************************************
static int _irq_enable(dev_data_t* dev, s32* arg)
{
	int	ret	= -EINVAL;

	if (arg[0] == -1)
	{
		ret		= 0;
		arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_pstsr_32);
		arg[0]	&= HSAI4_IRQ_ALL;
	}
	else if ((arg[0] & HSAI4_IRQ_ALL) == arg[0])
	{
		ret	= 0;
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_pstsr_32, arg[0]);
	}

	return(ret);
}



//*****************************************************************************
static int _xcvr(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSAI4_XCVR_TTL,
		HSAI4_XCVR_LVDS,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 7, 7);
	return(ret);
}



//*****************************************************************************
static int _rx_io_abort(dev_data_t* dev, s32* arg)
{
	arg[0]	= gsc_read_abort_active_xfer(dev, &dev->io.rx);
	return(0);
}



// variables ******************************************************************

const gsc_ioctl_t	dev_ioctl_list[]	=
{
	{ HSAI4_IOCTL_REG_READ,				(void*) gsc_reg_read_ioctl		},
	{ HSAI4_IOCTL_REG_WRITE,			(void*) gsc_reg_write_ioctl		},
	{ HSAI4_IOCTL_REG_MOD,				(void*) gsc_reg_mod_ioctl		},
	{ HSAI4_IOCTL_QUERY,				(void*) _query					},
	{ HSAI4_IOCTL_INITIALIZE,			(void*) initialize_ioctl		},
	{ HSAI4_IOCTL_AUTOCAL,				(void*) _autocal				},
	{ HSAI4_IOCTL_AUTOCAL_STATUS,		(void*) _autocal_status			},
	{ HSAI4_IOCTL_RX_IO_MODE,			(void*) _rx_io_mode				},
	{ HSAI4_IOCTL_RX_IO_OVERFLOW,		(void*) _rx_io_overflow			},
	{ HSAI4_IOCTL_RX_IO_TIMEOUT,		(void*) _rx_io_timeout			},
	{ HSAI4_IOCTL_RX_IO_UNDERFLOW,		(void*) _rx_io_underflow		},
	{ HSAI4_IOCTL_AI_BUF_CLEAR,			(void*) _ai_buf_clear			},
	{ HSAI4_IOCTL_AI_BUF_ENABLE,		(void*) _ai_buf_enable			},
	{ HSAI4_IOCTL_AI_BUF_LEVEL,			(void*) _ai_buf_level			},
	{ HSAI4_IOCTL_AI_BUF_OVERFLOW,		(void*) _ai_buf_overflow		},
	{ HSAI4_IOCTL_AI_BUF_THR_LVL,		(void*) _ai_buf_thr_lvl			},
	{ HSAI4_IOCTL_AI_BUF_THR_STS,		(void*) _ai_buf_thr_sts			},
	{ HSAI4_IOCTL_AI_BUF_UNDERFLOW,		(void*) _ai_buf_underflow		},
	{ HSAI4_IOCTL_AI_BURST_MARKER,		(void*) _ai_burst_marker		},
	{ HSAI4_IOCTL_AI_BURST_MODE,		(void*) _ai_burst_mode			},
	{ HSAI4_IOCTL_AI_BURST_SINGLE,		(void*) _ai_burst_single		},
	{ HSAI4_IOCTL_AI_BURST_SIZE,		(void*) _ai_burst_size			},
	{ HSAI4_IOCTL_AI_BURST_STATUS,		(void*) _ai_burst_status		},
	{ HSAI4_IOCTL_AI_BURST_TRIGGER,		(void*) _ai_burst_trigger		},
	{ HSAI4_IOCTL_AI_CHAN_SEL,			(void*) _ai_chan_sel			},
	{ HSAI4_IOCTL_AI_CHAN_TAG,			(void*) _ai_chan_tag			},
	{ HSAI4_IOCTL_AI_CLOCK_SRC,			(void*) _ai_clock_src			},
	{ HSAI4_IOCTL_AI_CLOCKING,			(void*) _ai_clocking			},
	{ HSAI4_IOCTL_AI_DATA_PACKING,		(void*) _ai_data_packing		},
	{ HSAI4_IOCTL_AI_MODE,				(void*) _ai_mode				},
	{ HSAI4_IOCTL_AI_PRETRIG_MODE,		(void*) _ai_pretrig_mode		},
	{ HSAI4_IOCTL_AI_PRETRIG_SIZE,		(void*) _ai_pretrig_size		},
	{ HSAI4_IOCTL_AI_PRETRIG_STS,		(void*) _ai_pretrig_sts			},
	{ HSAI4_IOCTL_AI_RANGE,				(void*) _ai_range				},
	{ HSAI4_IOCTL_AI_SOL_CHAN,			(void*) _ai_sol_chan			},
	{ HSAI4_IOCTL_AI_SOL_HOLDOFF,		(void*) _ai_sol_holdoff			},
	{ HSAI4_IOCTL_AI_SOL_INT,			(void*) _ai_sol_int				},
	{ HSAI4_IOCTL_AI_SOL_LEVEL,			(void*) _ai_sol_level			},
	{ HSAI4_IOCTL_AI_SYNC_SRC,			(void*) _ai_sync_src			},
	{ HSAI4_IOCTL_CBL_CLK_SYNC_DIR,		(void*) _cbl_clk_sync_dir		},
	{ HSAI4_IOCTL_DATA_FORMAT,			(void*) _data_format			},
	{ HSAI4_IOCTL_GEN_INT_CLK_STATUS,	(void*) _gen_int_clk_status		},
	{ HSAI4_IOCTL_GEN_NDEC,				(void*) _gen_ndec				},
	{ HSAI4_IOCTL_GEN_NDIV_CLK,			(void*) _gen_ndiv_clk			},
	{ HSAI4_IOCTL_GEN_NDIV_SYNC,		(void*) _gen_ndiv_sync			},
	{ HSAI4_IOCTL_GEN_NREF,				(void*) _gen_nref				},
	{ HSAI4_IOCTL_GEN_NVCO,				(void*) _gen_nvco				},
	{ HSAI4_IOCTL_IRQ_ENABLE,			(void*) _irq_enable				},
	{ HSAI4_IOCTL_XCVR,					(void*) _xcvr					},
	{ HSAI4_IOCTL_RX_IO_ABORT,			(void*) _rx_io_abort			},
	{ HSAI4_IOCTL_WAIT_EVENT,			(void*) gsc_wait_event_ioctl	},
	{ HSAI4_IOCTL_WAIT_CANCEL,			(void*) gsc_wait_cancel_ioctl	},
	{ HSAI4_IOCTL_WAIT_STATUS,			(void*) gsc_wait_status_ioctl	},
	{ -1, NULL }
};


