// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/14HSAI4/utils/util_config_ai.c $
// $Rev: 53627 $
// $Date: 2023-08-29 09:30:57 -0500 (Tue, 29 Aug 2023) $

// 14HSAI4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hsai4_config_ai
*
*	Purpose:
*
*		Configure the given device's analog input options.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	Aggregate sample rate.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int hsai4_config_ai(int fd, int index, int verbose, s32 fsamp)
{
	int			errs		= 0;
	s32			config;
	s32			ndec		= 1;
	s32			ndiv_clk	= 5;
	s32			nref		= 60;
	s32			nvco		= 50;
	s32			range;
	gsc_wait_t	wait;

	if (verbose)
	{
		gsc_label_index("Input Configuration", index);
		printf("\n");
		gsc_label_level_inc();
	}

	memset(&wait, 0xFF, sizeof(wait));

	errs	+= hsai4_initialize			(fd, index, verbose);

	// Settings must be applied after the initialization call!

	errs	+= hsai4_rx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,				NULL);
	errs	+= hsai4_rx_io_overflow		(fd, index, verbose, HSAI4_IO_OVERFLOW_CHECK,		NULL);
	errs	+= hsai4_rx_io_timeout		(fd, index, verbose, HSAI4_IO_TIMEOUT_DEFAULT,		NULL);
	errs	+= hsai4_rx_io_underflow	(fd, index, verbose, HSAI4_IO_UNDERFLOW_CHECK,		NULL);
	errs	+= hsai4_data_format		(fd, index, verbose, HSAI4_DATA_FORMAT_OFF_BIN,		NULL);
	errs	+= hsai4_xcvr				(fd, index, verbose, HSAI4_XCVR_TTL,				NULL);
	errs	+= hsai4_ai_mode			(fd, index, verbose, HSAI4_AI_MODE_SYSTEM,			NULL);
	errs	+= hsai4_ai_buf_thr_lvl		(fd, index, verbose, 128L * 1024,					NULL);
	errs	+= hsai4_ai_chan_sel		(fd, index, verbose, 0xF,							NULL);
	errs	+= hsai4_ai_chan_tag		(fd, index, verbose, HSAI4_AI_CHAN_TAG_ENABLE,		NULL);
	errs	+= hsai4_ai_clock_src		(fd, index, verbose, HSAI4_AI_CLOCK_SRC_INTERNAL,	NULL);
	errs	+= hsai4_ai_data_packing	(fd, index, verbose, HSAI4_AI_DATA_PACKING_DISABLE,	NULL);
	errs	+= hsai4_ai_pretrig_mode	(fd, index, verbose, HSAI4_AI_PRETRIG_MODE_DISABLE,	NULL);
	errs	+= hsai4_ai_pretrig_size	(fd, index, verbose, 1024,							NULL);
	errs	+= hsai4_ai_pretrig_sts		(fd, index, verbose,								NULL);

	errs	+= hsai4_query				(fd, index, verbose, HSAI4_QUERY_INPUT_CONFIG, &config);

	switch (config)
	{
		default:
		case HSAI4_INPUT_CONFIG_HIGH_LEVEL:		range	= HSAI4_AI_RANGE_10V;		break;
		case HSAI4_INPUT_CONFIG_WIDEBAND:		range	= HSAI4_AI_RANGE_2_5V;		break;
		case HSAI4_INPUT_CONFIG_TRANSFORMER:	range	= HSAI4_AI_RANGE_2_0VPP;	break;
	}

	errs	+= hsai4_ai_range			(fd, index, verbose, range,							NULL);

	errs	+= hsai4_ai_sol_chan		(fd, index, verbose, 0,								NULL);
	errs	+= hsai4_ai_sol_holdoff		(fd, index, verbose, HSAI4_AI_SOL_HOLDOFF_8,		NULL);
	errs	+= hsai4_ai_sol_int			(fd, index, verbose, HSAI4_AI_SOL_INT_1,			NULL);
	errs	+= hsai4_ai_sol_level		(fd, index, verbose, 0x3000,						NULL);
	errs	+= hsai4_gen_int_clk_status	(fd, index, verbose,								NULL);
	errs	+= hsai4_fsamp_ai_compute	(fd, index, verbose, fsamp, &nvco, &nref, &ndiv_clk, &ndec, NULL);
	errs	+= hsai4_gen_nvco			(fd, index, verbose, nvco,							NULL);
	errs	+= hsai4_gen_nref			(fd, index, verbose, nref,							NULL);
	errs	+= hsai4_gen_ndiv_clk		(fd, index, verbose, ndiv_clk,						NULL);
	errs	+= hsai4_gen_ndec			(fd, index, verbose, ndec,							NULL);
	errs	+= hsai4_ai_burst_mode		(fd, index, verbose, HSAI4_BURST_MODE_DISABLE,		NULL);
	errs	+= hsai4_ai_sync_src		(fd, index, verbose, HSAI4_AI_SYNC_SRC_INTERNAL,	NULL);
	errs	+= hsai4_cbl_clk_sync_dir	(fd, index, verbose, HSAI4_CBL_CLK_SYNC_DIR_OUTPUT,	NULL);
	errs	+= hsai4_ai_clocking		(fd, index, verbose, HSAI4_AI_CLOCKING_ENABLE,		NULL);
	errs	+= hsai4_ai_buf_enable		(fd, index, verbose, HSAI4_AI_BUF_ENABLE_YES,		NULL);
	errs	+= hsai4_fsync_ai_get		(fd, index, verbose,								NULL);

	errs	+= hsai4_fsamp_ai_report_all(fd, index, verbose,								NULL);

	// Settings affecting the sample rate or voltage range must generally
	// be applied before autocalibration.
	// Refer to the device user manual for clarification.
	errs	+= hsai4_autocal			(fd, index, verbose);	// Cable may need disconnecting for this.

	// Interrupt selections should be applied after autocalibration as the
	// driver overwrites the current selection to detect the Autocalibration
	// Done interrupt. If an application intends to detect the Autocalibration
	// Done interrupt, then the interrupt selection may be made beforehand.
	errs	+= hsai4_irq_enable			(fd, index, verbose, 0,								NULL);

	// The final step is to clear the buffer. This service clears the FIFO and
	// resets the overflow and underflow status bits.
	errs	+= hsai4_ai_buf_clear		(fd, index, verbose);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


